@@ -1,4 +1,4 @@
-# Copyright 2004-2017 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2018 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -52,6 +52,8 @@ def renpy_pure(fn):
     return fn
 
 
+import pygame_sdl2
+
 from renpy.text.extras import ParameterizedText, filter_text_tags
 from renpy.text.font import register_sfont, register_mudgefont, register_bmfont
 from renpy.text.text import language_tailor
@@ -68,7 +70,7 @@ from renpy.display.screen import get_widget_properties
 from renpy.display.focus import focus_coordinates
 from renpy.display.predict import screen as predict_screen
 
-from renpy.display.image import image_exists, image_exists as has_image
+from renpy.display.image import image_exists, image_exists as has_image, list_images
 from renpy.display.image import get_available_image_tags, get_available_image_attributes, check_image_attributes, get_ordered_image_attributes
 
 from renpy.display.im import load_surface, load_image
@@ -122,6 +124,7 @@ renpy_pure("filter_text_tags")
 import time
 import sys
 import threading
+import fnmatch
 
 
 def public_api():
@@ -160,7 +163,7 @@ def public_api():
     check_text_tags
     map_event, queue_event, clear_keymap_cache
     const, pure, not_const
-    image_exists, has_image
+    image_exists, has_image, list_images
     get_available_image_tags, get_available_image_attributes, check_image_attributes, get_ordered_image_attributes
     load_image, load_surface
     profile_memory, diff_memory, profile_rollback
@@ -229,6 +232,9 @@ def can_rollback():
     Returns true if we can rollback.
     """
 
+    if not renpy.config.rollback_enabled:
+        return False
+
     return renpy.game.log.can_rollback()
 
 
@@ -488,7 +494,7 @@ def get_attributes(tag, layer=None):
     :doc: image_func
 
     Return a tuple giving the image attributes for the image with `tag`. If
-    the image is now showing, returns None.
+    the image is not showing, returns None.
 
     `layer`
         The layer to check. If None, uses the default layer for `tag`.
@@ -944,6 +950,7 @@ def display_menu(items,
                  **kwargs):
     """
     :doc: se_menu
+    :name: renpy.display_menu
     :args: (items, interact=True, screen="choice")
 
     This displays a menu to the user. `items` should be a list of 2-item tuples.
@@ -2524,6 +2531,27 @@ def cache_unpin(*args):
     renpy.store._cache_pin_set = renpy.store._cache_pin_set - new_pins
 
 
+def expand_predict(d):
+    """
+    :undocumented:
+
+    Use the fnmatch function to expland `d` for the purposes of prediction.
+    """
+
+    if not isinstance(d, basestring):
+        return [ d ]
+
+    if not "*" in d:
+        return [ d ]
+
+    if "." in d:
+        l = list_files(False)
+    else:
+        l = list_images()
+
+    return fnmatch.filter(l, d)
+
+
 def start_predict(*args):
     """
     :doc: image_func
@@ -2531,13 +2559,29 @@ def start_predict(*args):
     This function takes one or more displayables as arguments. It causes
     Ren'Py to predict those displayables during every interaction until
     the displayables are removed by :func:`renpy.stop_predict`.
+
+    If a displayable name is a string containing one or more \\*
+    characters, the asterisks are used as a wildcard pattern. If there
+    is at least one . in the string, the pattern is matched against
+    filenames, otherwise it is matched against image names.
+
+    For example::
+
+        $ renpy.start_predict("eileen *")
+
+    starts predicting all images with the name eileen, while::
+
+        $ renpy.start_predict("images/concert*.*")
+
+    matches all files starting with concert in the images directory.
     """
 
     new_predict = renpy.python.RevertableSet(renpy.store._predict_set)
 
-    for d in args:
-        d = renpy.easy.displayable(d)
-        new_predict.add(d)
+    for i in args:
+        for d in expand_predict(i):
+            d = renpy.easy.displayable(d)
+            new_predict.add(d)
 
     renpy.store._predict_set = new_predict
 
@@ -2548,13 +2592,16 @@ def stop_predict(*args):
 
     This function takes one or more displayables as arguments. It causes
     Ren'Py to stop predicting those displayables during every interaction.
+
+    Wildcard patterns can be used as described in :func:`renpy.start_predict`.
     """
 
     new_predict = renpy.python.RevertableSet(renpy.store._predict_set)
 
-    for d in args:
-        d = renpy.easy.displayable(d)
-        new_predict.discard(d)
+    for i in args:
+        for d in expand_predict(i):
+            d = renpy.easy.displayable(d)
+            new_predict.discard(d)
 
     renpy.store._predict_set = new_predict
 
@@ -2604,6 +2651,11 @@ def call_screen(_screen_name, *args, **kwargs):
 
     renpy.exports.mode('screen')
 
+    with_none = renpy.config.implicit_with_none
+
+    if "_with_none" in kwargs:
+        with_none = kwargs.pop("_with_none")
+
     show_screen(_screen_name, _transient=True, *args, **kwargs)
 
     roll_forward = renpy.exports.roll_forward_info()
@@ -2615,11 +2667,6 @@ def call_screen(_screen_name, *args, **kwargs):
 
     renpy.exports.checkpoint(rv)
 
-    with_none = renpy.config.implicit_with_none
-
-    if "_with_none" in kwargs:
-        with_none = kwargs.pop("_with_none")
-
     if with_none:
         renpy.game.interface.do_with(None, None)
 
@@ -3441,3 +3488,44 @@ def get_texture_size():
     """
 
     return renpy.display.draw.get_texture_size()
+
+
+old_battery = False
+
+
+def get_on_battery():
+    """
+    :other:
+
+    Returns True if Ren'Py is running on a device that is powered by an internal
+    battery, or False if the device is being charged by some external source.
+    """
+
+    global old_battery
+
+    pi = pygame_sdl2.power.get_power_info()  # @UndefinedVariable
+
+    if pi.state == pygame_sdl2.POWERSTATE_UNKNOWN:  # @UndefinedVariable
+        return old_battery
+    elif pi.state == pygame_sdl2.POWERSTATE_ON_BATTERY:  # @UndefinedVariable
+        old_battery = True
+        return True
+    else:
+        old_battery = False
+        return False
+
+
+def get_say_image_tag():
+    """
+    :doc: image_func
+
+    Returns the tag corresponding to the currently speaking character (the
+    `image` argument given to that character). Returns None if no character
+    is speaking or the current speaking character does not have a corresponding
+    image tag.
+    """
+
+    if renpy.store._side_image_attributes is None:
+        return None
+
+    return renpy.store._side_image_attributes[0]
